; -----------------------------------------------------------------------------
; "Rotatrix" by Einar Saukas
; A Rainbow Graphics utility for rotating images outside the screen
; -----------------------------------------------------------------------------

; Rotating speed
IMAGE_SPEED     EQU  1

; Image address
IMAGE_ADDRESS   EQU  64000

; -----------------------------------------------------------------------------
        org     64859

main_routine:
; preserve all registers
        push    af
        push    bc
        push    de
        push    hl

; adjust timing
delays:
        ld      b, 133                  ; "LD B,136" if Spectrum 48K (adds 66 T-states)
        nop                             ; "LD A,(HL)" if Spectrum +2A/+3 (adds 3 T-states)
        add     hl, hl                  ; additional delay
        defb    $ed
        nop                             ; "LD A,I" if late timing (adds 1 T-state)
delay1:
        ld      a, i
        djnz    delay1
delay2:
        djnz    delay2

; calculate image addresses
        ld      b, 24                   ; 24 lines
        ld      hl, IMAGE_ADDRESS+49
next_image_addr:
        ld      d, (hl)
        dec     hl
        ld      e, (hl)                 ; image offset
        dec     hl
        push    hl
        add     hl, de                  ; image address
shift_image_addr:
        ld      de, 0
        add     hl, de                  ; shifted image address
        ex      (sp), hl
        djnz    next_image_addr

; additional parameters
        ld      d, (hl)                 ; number of columns
        dec     hl
        ld      e, (hl)                 ; default border color

; generate border images
        ld      bc, $20fe
        jr      loop
char_rotation:
        nop
        nop
loop:
        nop
        pop     hl
        inc     hl
REPT 12
        outi
ENDR
        jr      nz, loop-1              ; "JR NZ,loop" if Spectrum 48K (reduces 4 T-states)
image_finished:

; restore default border color
        out     (c),e

; rotate image
        ld      hl, counter+1
counter:
        ld      a, 0
        inc     (hl)
        cp      IMAGE_SPEED
        jr      c, done
        xor     a
        ld      (hl),a
        ld      hl, char_rotation-1
        ld      a, (hl)
        inc     (hl)
        cp      3
        jr      c, done
        xor     a
        ld      (hl),a
        ld      hl, shift_image_addr+1
        ld      a, (hl)
        inc     (hl)
        cp      d
        jr      c, done
        xor     a
        ld      (hl),a
done:

; restore all registers
        pop     hl
        pop     de
        pop     bc
        pop     af
        jp      $38

; -----------------------------------------------------------------------------
; "call 64974" to activate
        di

; Spectrum 48K settings
        ld      hl, 136
        ld      c, 227

; Spectrum 48K ROM?
        ld      a, ($004b)
        rra
        jr      c, configure

; Spectrum 128K settings
        ld      l, 133
        dec     c

; Spectrum 128K ROM?
        ld      a, ($0b53)
        rra
        jr      c, configure

; Spectrum +2A/+3 settings
        ld      h, $7e

; configure settings
configure:
        ld      (delays+1), hl
        ld      a, c
        ld      (image_finished-1), a

; activate interrupt routine
        ld      a, $fe
        ld      i, a
        im      2
        ei
        ret

; -----------------------------------------------------------------------------
; "call 65012" to deactivate
        di
        ld      a, $3f
        ld      i, a
        im      1
        ei
        ret

; -----------------------------------------------------------------------------
; interrupt address at $fdfd
        jp      main_routine

; -----------------------------------------------------------------------------
; jump vector table at addresses $fe00-$ff00
REPT 257
        defb    $fd
ENDR

; -----------------------------------------------------------------------------
